%
% Estimation of alternative indices, pooling all Ks
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function vec = alternatives_Kindep(db,vect_K)

nb_K= length(vect_K);

% Calculer la moyenne dans l'echantillon
nn = zeros(length(vect_K),1);
moy = zeros(length(vect_K),1);
for j = 1:length(vect_K)
    K=vect_K(j);
    [val,iiii] = min(db');
    count = db( (iiii-2 == K)&(val==-99) ,:);
    count = count(count>-99);
    nn(j) = sum(count);
    moy(j)=sum((count.*(0:K))/(K*nn(j)));
end
m1=sum(moy.*nn)/sum(nn);

Theil_naive = zeros(nb_K,1); Duncan_naive=Theil_naive;
Theil_alea = zeros(nb_K,1); Duncan_alea=Theil_alea;
Theil_ABW_alea = zeros(nb_K,1); Duncan_ABW_alea=Theil_ABW_alea;

nn= zeros(nb_K,1);

for jj = 1:nb_K
    K=vect_K(jj);
    k=floor(K/2)+1;
   
    % Gere les donnees
    [val,iiii] = min(db');
    count = db( (iiii-2 == K)&(val==-99) ,:);
    count = count(count>-99);
    nn(jj) = sum(count);
    Pcomplet = count'/nn(jj);
    Pchap = Pcomplet(2:end);
    
    fT = inline('1-xlnx(x)/xlnx(m(1))','x','m'); 
    fD = inline('abs(x-m(1))/(2*m(1)*(1-m(1)))','x','m'); 
    
    % Naive
    Theil_naive(jj) = fT((0:K)/K, m1) *[1-sum(Pchap);Pchap];
    Duncan_naive(jj) = fD((0:K)/K, m1) *[1-sum(Pchap);Pchap];
    
    % Alea - CT
    vect_proba = choose(K,0:K).* (m1.^(0:K)).*((1-m1).^(K:-1:0));
    Theil_alea(jj) = vect_proba * fT((0:K)'/K, m1);
    Duncan_alea(jj) = vect_proba * fD((0:K)'/K, m1);
        
    %Bootstrap - ABW
    TT=50;
    TABWi = zeros(1,TT); DABWi=TABWi;
    Xalea = zeros(nn(jj),1);
    vect_cum = [0,cumsum(count)];
    for ii=1:TT
        for ll = 1:(K+1)
            Xalea(vect_cum(ll)+1:vect_cum(ll+1)) = binornd(K,(ll-1)/K,count(ll),1);
        end
        TABWi(ii) = mean(fT(Xalea/K, m1));
        DABWi(ii) = mean(fD(Xalea/K, m1));
    end
    Theil_ABW_alea(jj) = mean(TABWi);
    Duncan_ABW_alea(jj) = mean(DABWi);
    
end

Theil_naive_pooled = sum(Theil_naive.*nn)/sum(nn);
Duncan_naive_pooled = sum(Duncan_naive.*nn)/sum(nn);

Theil_CT_pooled = (Theil_naive_pooled- sum(Theil_alea.*nn)/sum(nn) )/(1-sum(Theil_alea.*nn)/sum(nn));
Duncan_CT_pooled = (Duncan_naive_pooled- sum(Duncan_alea.*nn)/sum(nn) )/(1-sum(Duncan_alea.*nn)/sum(nn));

Theil_ABW_pooled = 2*Theil_naive_pooled - sum(Theil_ABW_alea.*nn)/sum(nn);
Duncan_ABW_pooled = 2*Duncan_naive_pooled - sum(Duncan_ABW_alea.*nn)/sum(nn);

vec = [Theil_naive_pooled Duncan_naive_pooled Theil_CT_pooled Duncan_CT_pooled Theil_ABW_pooled Duncan_ABW_pooled];
