%
% Estimation of alternative indices, K by K
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function mat = alternatives_Kwise(db,vect_K)

nb_K= length(vect_K);

Theil_naive = zeros(nb_K,1);
Duncan_naive = zeros(nb_K,1);
Theil_CT = zeros(nb_K,1);
Duncan_CT = zeros(nb_K,1);
Theil_ABW = zeros(nb_K,1);
Duncan_ABW = zeros(nb_K,1);

for jj = 1:nb_K
    K=vect_K(jj);
    
    % Data manipulation
    [val,iiii] = min(db');
    count = db( (iiii-2 == K)&(val==-99) ,:);
    count = round(count(count>-99));
    n = round(sum(count));
    Pcomplet = count'/n;
    Pchap = Pcomplet(2:end);
    
    fT = inline('1-xlnx(x)/xlnx(m(1))','x','m'); 
    fD = inline('abs(x-m(1))/(2*m(1)*(1-m(1)))','x','m'); 
    
    % Naive
    m1K = ((1:K)/K)*Pchap;
    Theil_naive(jj) = fT((0:K)/K, m1K) *[1-sum(Pchap);Pchap];
    Duncan_naive(jj) = fD((0:K)/K, m1K) *[1-sum(Pchap);Pchap];
    
    % Carrington-Troske 
    vect_proba = choose(K,0:K).* (m1K.^(0:K)).*((1-m1K).^(K:-1:0));
    Talea = vect_proba * fT((0:K)'/K, m1K);
    Dalea = vect_proba * fD((0:K)'/K, m1K);

    Theil_CT(jj) = (Theil_naive(jj) - Talea)/(1-Talea);
    Duncan_CT(jj) = (Duncan_naive(jj) - Dalea)/(1-Dalea);
    
    % Allen et al. 
    TT = 50;
    TABWi = zeros(1,TT); DABWi=TABWi;
    Xalea = zeros(n,1);
    vect_cum = [0,cumsum(count)];
    for ii=1:TT
        for ll = 1:(K+1)
            Xalea(vect_cum(ll)+1:vect_cum(ll+1)) = binornd(K,(ll-1)/K,count(ll),1);
        end
        TABWi(ii) = mean(fT(Xalea/K, m1K));
        DABWi(ii) = mean(fD(Xalea/K, m1K));
    end
    Theil_ABW(jj)= 2*Theil_naive(jj)-mean(TABWi);
    Duncan_ABW(jj)= 2*Duncan_naive(jj)-mean(DABWi);
end

mat = [Theil_naive Duncan_naive Theil_CT Duncan_CT Theil_ABW Duncan_ABW];
