%
% Estimation of the bounds
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function [boundsW,boundsU,Pstacke,contraintstacke,Theilstacke,xpout] = bounds(db, vect_K, xpmat)

% Estimation of m01, Pchap, mchap
nn = zeros(length(vect_K),1);
mm1 = zeros(length(vect_K),1);
Pchapmat = zeros(length(vect_K),max(vect_K));
xpout = zeros(length(vect_K),2*(floor(length(vect_K)/2)+1));
contraintmat = zeros(length(vect_K),1);

for j = 1:length(vect_K)
    % data manipulation
    K=vect_K(j);
    k=floor(K/2)+1;
    [Ptilde, n] = fct_datamanip(db,K);
    % Default value
    if ~exist('xpmat', 'var')
        xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;
    else
        xp0 = xpmat(j,1:(2*k))';
    end
    % constrained ML
    [Pchap,contraint,xp1] = fct_Pchap_CML(Ptilde,xp0);
    % store for later
    Pchapmat(j,1:K) = Pchap;
    xpout(j,1:length(xp1)) = xp1; 
    contraintmat(j) = contraint;
    % compute the moments
    mchap = fct_Q(K) \ Pchap;
    mm1(j) = mchap(1);
    nn(j)=n;
end
m1=sum(mm1.*nn.*vect_K')/sum(nn.*vect_K');

% Bounds estimation
mat = zeros(length(vect_K),5);
Pstacke = zeros(sum(vect_K),1);
contraintstacke = zeros(length(vect_K),1);
compteur = 1 ;

for j = 1:length(vect_K)
    K=vect_K(j);
    k=floor(K/2)+1;
    
    % Constrained ML
    Pchap = Pchapmat(j,1:K)';
    xp1 = xpout(j,1:(2*k))';
    contraint = contraintmat(j);
    Pstacke(compteur:compteur+K-1) = Pchapmat(j,1:K);
    mchap = fct_Q(K) \ Pchap;
    
    % Bounds 
    bornes_Theil = estimation_bornes_Theil_cheb_Krandom(mchap, xp1, contraint, m1);
    bornes_Duncan = estimation_bornes_Duncan_nonlin_Krandom(mchap, xp1, contraint, m1);
    
    mat(j,:) = [1-contraint bornes_Theil' bornes_Duncan'];
    contraintstacke(j,1) = contraint;
    compteur=compteur+K;
    
end

% Bounds, with weights
boundsW = ((nn.*vect_K')'/sum(nn.*vect_K'))*mat;
% Bounds, without weights
boundsU = (nn'/sum(nn))*mat;
Theilstacke = mat(:,2:3);
    
end
