%
% Estimation of the bounds and bootstrap (one line for each simulation)
% Case: K random, not necessarily independent from p
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function bornes_and_CI = boundsCI(db, vect_K, nboot)

    % Clean the data
    [db, vect_K] = fct_dataselect(db,vect_K);
    
    % compute bounds
    [bornesW,bornesU,Pstacke,contraintstacke,Theilstacke,xpmat] = bounds(db, vect_K);
    
    % Total for each K
    nK=zeros(length(vect_K),1);
    for kk=1:length(vect_K)
        [~, nK(kk)] = fct_datamanip(db,vect_K(kk));
    end

    % Computation of the P used in the bootstrap
    compteurP = 1 ;
    Pb=zeros(max(vect_K)+1,length(vect_K));
    for kk=1:length(vect_K)
        K = vect_K(kk);
        k=floor(K/2)+1;
        xp1 = xpmat(kk,1:(2*k))';
        Pchap = Pstacke(compteurP:compteurP+K-1);
        Pcomplet = [1-sum(Pchap); Pchap];
        if contraintstacke(kk) == 0
            % Interior case: Use Pbootstrap (dealing with false interior cases)
            Pb(1:K+1,kk) = Pchap_to_Pbootstrap(Pcomplet,nK(kk),Theilstacke(kk,:),xp1);    
        else
            % Exterior case: directly use Pcomplet
            Pb(1:K+1,kk) = Pcomplet;
        end
        compteurP= compteurP + K ;
    end
    
    % Bootstrap iterations
    bornesW_boot=zeros(nboot,length(bornesW));
    bornesU_boot=zeros(nboot,length(bornesU));    
    for jj = 1:nboot
        % Resample same size 
        db_boot = db*0-99;
        % Start with resampling K in its empirical distribution
        vect_nk = mnrnd(sum(nK),nK/sum(nK));
        compteurP = 1 ;
        % Then, resample X|K 
        for kk=1:length(vect_K)
            K = vect_K(kk);
            count_boot = mnrnd(vect_nk(kk),Pb(1:K+1,kk));
            db_boot(kk,1:length(count_boot))=count_boot;
            compteurP= compteurP + K ;
        end
        % Estimation of the bounds
        [bornesW_boot(jj,:),bornesU_boot(jj,:)] = bounds(db_boot, vect_K, xpmat);
    end
    
    % Compute global distance to frontier (using Theil indices)
    distance_frontier = (bornesW(3)-bornesW(2))/...
        (sqrt(2*log(log(sum(nK)))) * std(bornesW_boot(:,3)-bornesW_boot(:,2)));
    
    if bornesW(2)==bornesW(3)
	distance_frontier = 0;
    end
        
    % Compute CI
    [bornesWIC,bornesWICint,bornesWICext] = fct_compute_CI(bornesW,bornesW_boot,distance_frontier);
    [bornesUIC,bornesUICint,bornesUICext] = fct_compute_CI(bornesU,bornesU_boot,distance_frontier);    
    bornes_and_CI = [bornesW bornesWIC bornesWICext bornesWICint distance_frontier; 
        bornesU bornesUIC bornesUICext bornesUICint distance_frontier];
    
end
