%
% Estimation of the bounds of the Duncan index (regular method), pooling Ks
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function bornes = estimation_bornes_Duncan_nonlin_Krandom(m, xp_constr, contraint,m1)

% Unconstrained case
if contraint ==0

    K = length(m);
    k=floor(K/2);

    % We first seek feasible starting values, using Curto and Fialkow (1991)

    % Odd case
    if 2*k<K
        vr_r_1 = m(k+1:end);
        m_c =[1;m];
        A_r_1 = hankel(m_c(1:k+1),m_c(k+1:2*k+1));
        Phi_tilde = A_r_1 \ vr_r_1 ;

        coeffs_poly = [1;-flipud(Phi_tilde)];

        x= roots(coeffs_poly);
        Ax = repmat(x',K+1,1).^(repmat((0:K)',1,k+1));
        p = Ax \ m_c;

        xp0_init = [x;p];

    % Even case
    else

        m_c_bis =[m;0];
        V = hankel(m_c_bis(1:k+1),m_c_bis(k+1:2*k+1));
        V=V(:,1:end-1);
        V1 = V(1:end-1,:);
        V2 = V(end,:);
        lambda = V1 \m(k+1:end);
        c = V2 * lambda;

        m_c =[1;m;c];
        A_r_1 = hankel(m_c(1:k+1),m_c(k+1:2*k+1));
        vr_r_1 = m_c(k+2:end);
        Phi_tilde = A_r_1 \ vr_r_1 ;

        coeffs_poly = [1;-flipud(Phi_tilde)];

        x= roots(coeffs_poly);
        Ax = repmat(x',K+2,1).^(repmat((0:K+1)',1,k+1));
        p = Ax \ m_c;

        xp0_init = [x;p];
    end


    Abis = [diag(ones(K,1),-1)-eye(K+1);[zeros(1,K),1]];
    A = [[Abis,zeros(K+2,K+1)];[zeros(K+1),-eye(K+1)]];
    b=[zeros(K+1,1);1;zeros(K+1,1)];

    Aeq = [zeros(1,K+1),ones(1,K+1)];
    beq = 1;

    % Note: for some unknown reason, the optimization does not seem to work
    % well when GradObj is put to 'on'.
    options = optimset(optimset('fmincon'),'Display','off','GradObj','off','Algorithm','sqp','MaxFunEvals',2e5,'MaxIter',2e5);

    bornes=zeros(2,1);

    exitinf = 0; 

    while exitinf<1

        temp = reshape(xp0_init,k+1,2);
        temp2 = [temp;[rand(K-k,1),zeros(K-k,1)]];
        xp0 = reshape(sortrows(temp2),2*(K+1),1);
        xp0 = max(xp0,0);
        [xp_sol_min, bornes(1),exitinf] = fmincon(@(xp) Duncan(xp,m1,1),xp0,A,b,Aeq,beq,[],[],@(xp) contraintes(xp,m),options);
        xp_sol_min;
    end


    exitsup = 0;

    while exitsup <1

        temp = reshape(xp0_init,k+1,2);
        temp2 = [temp;[rand(K-k,1),zeros(K-k,1)]];
        xp0 = reshape(sortrows(temp2),2*(K+1),1);
        xp0 = max(xp0,0);

        [xp_sol_max, bornes(2),exitsup] = fmincon(@(xp) Duncan(xp,m1,0),xp0,A,b,Aeq,beq,[],[],@(xp) contraintes(xp,m),options);
        bornes(2)=-bornes(2);
        xp_sol_max;
    end

    % Constrained case
else
       bornes = repmat(borne_Duncan(xp_constr, m1, 1),2,1);
end
    
    
end


function [f,g] = Duncan(xp,m1,indic_min)

x = xp(1:end/2);
p = xp(end/2+1:end);

if indic_min==1
    f= (abs(x-m1)/(2*m1*(1-m1)))'  * p;
else
    f= -(abs(x-m1)/(2*m1*(1-m1)))'  * p;
end

end

function [c,ceq] = contraintes(xp,m)

c = 0;

K = length(xp)/2 -1 ;
x = xp(1:end/2);
p = xp(end/2+1:end);
v = (1:K);
Ax = repmat(x',K,1).^(repmat(v',1,K+1));

% B = 1./(repmat(v,K,1)+ repmat(v',1,K)+1);
% Mat_orth = sqrtm(inv(B));

ceq =  Ax * p - m;

end