%
% Bootstrap CI of the bounds of the Duncan and Theil indices
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
%    

function mat = fct_CI1(Pchap2_complet,xp1,contraint,n,Q,bornes_Theil,bornes_Duncan,nb_boot,alp,indic_theilchebyshev)
    
    % Redefinition of the bounds
    borne_inf_Theil = bornes_Theil(1);
    borne_sup_Theil = bornes_Theil(2);
    borne_inf_Duncan = bornes_Duncan(1);
    borne_sup_Duncan = bornes_Duncan(2);
    
    indic_frontiere = contraint ;

    % Il faut grer le cas "faux intrieur" o on est proche du bord
    if contraint == 0    
        
        % On calcule d'abord une "distance"  la frontire base sur
        % borne_Theil_sup - borne_Theil_inf. On fait qqs bootstrap pr
        % calculer l'cart-type de cette variable.
        nb_boot_init = 20 ;
        
        bornes_Theil_boot_init = zeros(2,nb_boot_init);
        indic_interieur = 0;
        bb = 0;
        
        % Double critre : au moins 20 itrations ET au moins une fois 
        % l'intrieur
        while bb < nb_boot_init  || indic_interieur ==0 
            
            count_boot = mnrnd(n,Pchap2_complet);
            Pchap_boot = count_boot(2:end)'/n;
            mchap_boot = Q \ Pchap_boot;
            contraint_boot = fct_contraint(mchap_boot);
            if contraint_boot == 1
                xp_boot = projection_xp(Pchap_boot, xp1);
                xp_boot = projection_maxvrais(Pchap_boot, xp_boot);
            else
                xp_boot = xp1;
                indic_interieur = 1 ;
            end
            
            bb = bb+1;
            if indic_theilchebyshev == 0
                bornes_Theil_boot_init(:,bb) = estimation_bornes_Theil_non_lin(mchap_boot, xp_boot, contraint_boot);
            else
                bornes_Theil_boot_init(:,bb) = estimation_bornes_Theil_cheb(mchap_boot, xp_boot, contraint_boot); 
            end
        end
        
        r_n = log(n);
        distance_frontiere = sqrt(n) * (bornes_Theil(2)-bornes_Theil(1))/...
            (r_n * std(bornes_Theil_boot_init(2,:)-bornes_Theil_boot_init(1,:))); 
        
        % Cas o on est proche du bord : on projete mchap sur la frontire
        if (distance_frontiere<10)
            indic_frontiere = 1;
            Pchap2_complet = proj_frontiere2(Pchap2_complet, n);
        end
        
    end


    % Now that the false interior case is settled, let's go for the bootstrap
    bornes_Theil_boot = zeros(2,nb_boot);
    bornes_Duncan_boot = zeros(2,nb_boot);

    for bb=1:nb_boot
        
        count_boot = mnrnd(n,Pchap2_complet);
        
        if count_boot(1)==n %gere le cas count_boot = [n 0 0 0...]
            bornes_Theil_boot(:,bb)=[0;1];
            bornes_Duncan_boot(:,bb)=[0;1];
        else
            Pchap_boot = count_boot(2:end)'/n;
            mchap_boot = Q \ Pchap_boot;
            contraint_boot = fct_contraint(mchap_boot);
            
            if contraint_boot == 1
                xp_boot = projection_xp(Pchap_boot, xp1);
            else
                xp_boot = xp1;
            end
            
            if indic_theilchebyshev == 1
                bornes_Theil_boot(:,bb) = estimation_bornes_Theil_cheb(mchap_boot, xp_boot, contraint_boot); 
            else
                bornes_Theil_boot(:,bb) = estimation_bornes_Theil_non_lin(mchap_boot, xp_boot, contraint_boot);
            end                
            bornes_Duncan_boot(:,bb) = estimation_bornes_Duncan_non_lin(mchap_boot, xp_boot, contraint_boot); 
        end
        
    end

    Theil_middle = (borne_inf_Theil+borne_sup_Theil)/2;
    Duncan_middle = (borne_inf_Duncan+borne_sup_Duncan)/2;

    if indic_frontiere == 1
        quant_Theil = quantile(abs(mean(bornes_Theil_boot,1) - Theil_middle) ,1-alp);
        quant_Duncan = quantile(abs(mean(bornes_Duncan_boot,1) - Duncan_middle) ,1-alp);
        borneIC_inf_Theil = Theil_middle - quant_Theil;
        borneIC_sup_Theil  = Theil_middle + quant_Theil;
        borneIC_inf_Duncan = Duncan_middle - quant_Duncan;
        borneIC_sup_Duncan  = Duncan_middle + quant_Duncan;
    else
        borneIC_inf_Theil  = 2 * borne_inf_Theil - quantile(bornes_Theil_boot(1,:),1-alp);
        borneIC_sup_Theil  = 2 * borne_sup_Theil - quantile(bornes_Theil_boot(2,:),alp);
        borneIC_inf_Duncan = 2 * borne_inf_Duncan - quantile(bornes_Duncan_boot(1,:),1-alp);
        borneIC_sup_Duncan = 2 * borne_sup_Duncan - quantile(bornes_Duncan_boot(2,:),alp);
    end     

    mat=[borneIC_inf_Duncan,borneIC_sup_Duncan,borneIC_inf_Theil,borneIC_sup_Theil];    

end 