%
% Cleans the dataset by keeping only the rows corresponding to the
% corresponding vector of K and the columns with some info
% All rows of a dataset should have a -99 value to close the row
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
%

function [dbcorr, vect_Kcorr] = fct_dataselect(db,vect_K)

    % Keep only rows with something in the first column
    db=db(db(:,1)>-99,:);
    
    % Check that each row has a -99 and where the first one is
    [val,iiii] = min(db,[],2);
    db=db(val==-99,:);
    
    % Infer the vector of K in the data
    vecK = iiii-2;
    
    % Locate the relevant K in the data
    [ism,loc] = ismember(vect_K,vecK); 
    
    % Restrict the vector of K to those in the data
    vect_Kcorr = vect_K(ism);
    
    % Restrict the data to the relevant Ks
    % Eliminate the columns that are not useful
    dbcorr = db(loc(ism),1:(max(vect_Kcorr)+2));
   
end