%
% Projection of the estimated P on the space corresponding to \mathcal M
%   using maximum likelihood
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

function  [xp1,Pchap2_complet,exit] = projection_maxvrais(Pchap, xp0)

    opts4 = optimset(optimset('fmincon'),'Display','off','MaxIter',2000,'MaxFunEvals',50000,'TolCon',1e-7,'Algorithm','sqp');

    K = length(Pchap);
    Q = fct_Q(K);
    k = length(xp0)/2;

    % Pour taille k
    % Contraintes sur les x : suite croissante, positive et <= 1
    % + contraites sur les p : p>0
    Ad = [ [[-1,zeros(1,k-1)];eye(k) - diag(ones(k-1,1),1)] zeros(k+1,k); zeros(k,k) (-eye(k))]; 
    bd = [[-1e-6;-2e-2 * ones(k-1,1);1]; -1e-6*ones(k,1)];
    % Contrainte sur les p : somment  1
    Adeq = [zeros(1,k) ones(1,k)];
    bdeq = 1;    

    P0 = 1-sum(Pchap);

    [xp1,~,exit] = fmincon(@(xp) logvrais(xp, Pchap, P0, Q, K, k), xp0, Ad, bd, Adeq, bdeq,[],[],[],opts4);

    Pchap2_complet = fct_Pcomplet(xp1,K);

end

function res = logvrais(xp, Pchap, P0, Q, K, k)

    x = xp(1:k);
    p = xp(k+1:end);
    A =  (ones(K,1) * x' ).^((1:K)' * ones(1,k));
    Q_m = Q * A * p;
    Q_m0 = 1- sum(Q_m);
    
    res = - (P0 * log(Q_m0) + Pchap' * log(Q_m)) ; 

end


