#
# Generate the graphs and the tables for the real data
# Updated May 2016
# Xavier d'Haultfoeuille and Roland Rathelot
#


## Import the set of functions that compute beta indices
source("fun_segregbeta.R")

## Clean data
dat <- read.csv("counts.csv")
dat <- as.matrix(dat)
dat <- dat[,-1]
dads <- cbind(dat,-99)

dat <- read.csv("counts0.csv")
dat <- as.matrix(dat)
dat <- dat[,-1]
dads0 <- cbind(dat,-99)

dat <- read.csv("counts1.csv")
dat <- as.matrix(dat)
dat <- dat[,-1]
dads1 <- cbind(dat,-99)


## Estimation K by K
Kvec <- 2:25
dads.ind.byK <- alternative.indices(dads,Kvec)
dads.ind.byK <- dads.ind.byK$byK

inddads <- list()
for (jj in 1:length(Kvec)){
    inddads[[jj]] <- betamix(dads,Kvec[jj],Cmax=1,deltameth=FALSE,bootstrap=FALSE)
}
dads.ind.byK$Tbeta <- do.call("c",lapply(inddads,function(vv){vv$indices[,,"T"]}))
dads.ind.byK$Dbeta <- do.call("c",lapply(inddads,function(vv){vv$indices[,,"D"]}))

## Pooling all K
bestim=betamix(dads,Kvec,Cmax=2,deltameth=FALSE,bootstrap=FALSE)
initv = list(bestim$estim[[1]]$par,bestim$estim[[2]]$par)


## Figure 3 of the paper
matlabres <- read.csv("data_Kwise_Fig3.csv",header=FALSE)
dimnames(matlabres)[[2]] <- c("K","Tinf","Tsup","Dinf","Dsup","TICinf","TICsup","DICinf","DICsup","TNaive","DNaive","TCT","DCT","TABW","DABW")

theilmat <- matlabres[,c("K","Tinf","Tsup","TICinf","TICsup","TNaive","TCT","TABW")]
dimnames(theilmat)[[2]] = c("K","binf","bsup","ICinf","ICsup","Naive","CT","ABW")
theilmat$Beta = dads.ind.byK$Tbeta

duncanmat <- matlabres[,c("K","Dinf","Dsup","DICinf","DICsup","DNaive","DCT","DABW")]
dimnames(duncanmat)[[2]] = c("K","binf","bsup","ICinf","ICsup","Naive","CT","ABW")
duncanmat$Beta = dads.ind.byK$Dbeta

mat=duncanmat
pdf("dads.D.pdf")
plot(mat[,1],mat[,"binf"],ylim=c(0,1),type="l",xlab="",ylab="")
lines(mat[,1],mat[,"bsup"],lty=1)
lines(mat[,1],mat[,"ICinf"],lty=2,col=1)
lines(mat[,1],mat[,"ICsup"],lty=2,col=1)
lines(mat[,1],mat[,"Naive"],lty=4,col=2)
lines(mat[,1],mat[,"CT"],lty=4,col=3)
lines(mat[,1],mat[,"ABW"],lty=4,col=4)
lines(mat[,1],mat[,"Beta"],lty=4,col="orange")
legend("bottomright",lty=c(1,2,4,4,4,4), col=c(1,1,2,3,4,"orange"), legend=(c("Identification bounds","CI","Naive","Carrington-Troske","Allen et al","Rathelot")))
dev.off()

mat=theilmat
pdf("dads.T.pdf")
plot(mat[,1],mat[,"binf"],ylim=c(0,1),type="l",xlab="",ylab="")
lines(mat[,1],mat[,"bsup"],lty=1)
lines(mat[,1],mat[,"ICinf"],lty=2,col=1)
lines(mat[,1],mat[,"ICsup"],lty=2,col=1)
lines(mat[,1],mat[,"Naive"],lty=4,col=2)
lines(mat[,1],mat[,"CT"],lty=4,col=3)
lines(mat[,1],mat[,"ABW"],lty=4,col=4)
lines(mat[,1],mat[,"Beta"],lty=4,col="orange")
legend("bottomright",lty=c(1,2,4,4,4,4), col=c(1,1,2,3,4,"orange"), legend=(c("Identification bounds","CI","Naive","Carrington-Troske","Allen et al","Rathelot")))
dev.off()
