%
% Analyses of the data: tables and figures of the paper
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 

%Load the data
clc
clear
dads=dlmread('counts.csv',',',1,0);
dads0=dlmread('counts0.csv',',',1,0);
dads1=dlmread('counts1.csv',',',1,0);
% add a column of zeros at the end
dads = [dads -99*ones(size(dads,1),1)];
dads0 = [dads0 -99*ones(size(dads0,1),1)];
dads1 = [dads1 -99*ones(size(dads1,1),1)];
% remove first column (K)
dads = dads(:,2:end);
dads0 = dads0(:,2:end);
dads1 = dads1(:,2:end);

%%% Table 4: test of the binomial model 
% For K<9, we get bounds so the test is automatically accepted
% (see Section C.1 in the online appendix) 
Kvec = 9:25;
nboot = 1e3;
res_test = zeros(length(Kvec),1);
for jj=1:length(Kvec);
    res_test(jj)=fct_test_binomiale_bootmv(dads,Kvec(jj),nboot);
end
dlmwrite('dads_binomtest_Tab4.csv', res_test, '-append','delimiter' , ',')  


%%% Figure 3: Theil and Duncan indices K-wise, using our methods and others
Kvec = 2:25;
nboot=200;
res1 = zeros(length(Kvec),9);
for jj=1:length(Kvec);
    %%% Estimated bounds and their CI
    tmp = boundsCI(dads,Kvec(jj),nboot);
    res1(jj,:) = [Kvec(jj) tmp(1,[2:5 7:10])];
end
%%% Alternative indices
resa = alternatives_Kwise(dads,Kvec);
reskwise = [res1 resa];
dlmwrite('data_Kwise_Fig3.csv', reskwise, '-append','delimiter' , ',')


%%% Table 5: equality tests of indices across sizes
Kvec = 2:9;
test_constant_segreg(dads, Kvec, 0) %Duncan
test_constant_segreg(dads, Kvec, 1) %Theil

Kvec = 10:25;
test_constant_segreg(dads, Kvec, 0)  %Duncan 
test_constant_segreg(dads, Kvec, 1)  %Theil

Kvec = 2:25;
test_constant_segreg(dads, Kvec, 0)  %Duncan 
test_constant_segreg(dads, Kvec, 1)  %Theil


%%% Table 6 
nboot=200;
Kvec = 2:25;

%%% (panel 1): Bounds and CI (K pooled), unconditional indices
bci = boundsCI(dads, Kvec, nboot);
bci(:,[2:5 7:10])

%%% (panel 1): Alternative (K pooled), unconditional indices
alternatives_Kindep_CI(dads, Kvec, nboot)

%%% (panel 2): Bounds and CI (K pooled), conditional indices (unskilled)
bci0 = boundsCI(dads0, Kvec, nboot);
bci0(:,[2:5 7:10])

%%% (panel 2): Bounds and CI (K pooled), conditional indices (skilled)
bci1 = boundsCI(dads1, Kvec, nboot);
bci1(:,[2:5 7:10])

bcidot = conditional_index(bci0,bci1,dads0,dads1);
