#
# Generate the graphs and the tables for the simulations
# Updated May 2016
# Xavier d'Haultfoeuille and Roland Rathelot
#


## Import the set of functions that compute beta indices
source("fun_segregbeta.R")

## To make things faster: moments of the simulated distributed
## (copied from Matlab)
momnorm = c(   0.046114198100539,
   0.018088787495892,
   0.010607548103868,
   0.007346802321176,
   0.005566330839362,
   0.004458977868911,
   0.003709329889199,
   0.003170696634727,
   0.002766236626939,
   0.002452047990966,
   0.002201331472523,
   0.001996850048181,
   0.001827038537087,
   0.001683861385098,
   0.001561569169572)

## Compute the dataset
n=1e5
Kvec = 2:10
db=matrix(-99,nrow=length(Kvec),ncol=max(Kvec)+2)
for (j in 1:length(Kvec)){
  K = Kvec[j]
  mat1K = matrix(1,nrow=K,ncol=1)%*%matrix(1:K,nrow=1,ncol=K)
  matm1=(-1)^(mat1K - t(mat1K))
  matcij=choose(mat1K,t(mat1K))
  matcjK=matrix(1,nrow=K,ncol=1)%*%choose(K,1:K)
  Q=matm1*matcij*matcjK
  Pvrai = c(1-sum(Q %*% momnorm[1:K]), t(Q%*%momnorm[1:K]))
  db[j,1:(K+1)] = n*Pvrai                    ## True P
}

## Pooling all K
bestim=betamix(db,Kvec,Cmax=2,deltameth=FALSE,bootstrap=FALSE)

## K-wise, taking pooled values as starting values
initpar = bestim$estim[[2]]$par
Hind=rep(0,length(Kvec))
Dind=rep(0,length(Kvec))
for (j in 1:length(Kvec)){
    res=betamix(db[j,],Kvec[j],Cmax=2,initval=list(NULL,initpar),deltameth=FALSE,bootstrap=FALSE)
    Hind[j]=res$ind[,,"T"][length(res$ind[,,"T"])]
    Dind[j]=res$ind[,,"D"][length(res$ind[,,"D"])]
}

## Figure 1: Identification bounds and alternative estimators
matlabres <- read.csv("simu_Kwise_Fig1.csv",header=FALSE)
hh <- matlabres[,c(1:3,6,8,10)]
dimnames(hh)[[2]] = c("K","binf","bsup","Naive","CT","ABW")
hh$Beta = Hind
dd <- matlabres[,c(1,4:5,7,9,11)]
dimnames(dd)[[2]] = c("K","binf","bsup","Naive","CT","ABW")
dd$Beta = Dind

mat=dd
pdf("comparison.D.norm.pdf")
plot(mat[,1],mat[,"binf"],ylim=c(0,1),type="l",xlab="",ylab="")
lines(mat[,1],mat[,"bsup"],lty=1)
lines(mat[,1],mat[,"Naive"],lty=4,col=2)
lines(mat[,1],mat[,"CT"],lty=4,col=3)
lines(mat[,1],mat[,"ABW"],lty=4,col=4)
lines(mat[,1],mat[,"Beta"],lty=4,col="orange")
legend("bottomright",lty=c(1,4,4,4,4), col=c(1,2,3,4,"orange"), legend=(c("Identification bounds","Naive","Carrington-Troske","Allen et al","Rathelot")))
dev.off()

mat=hh
pdf("comparison.T.norm.pdf")
plot(mat[,1],mat[,"binf"],ylim=c(0,1),type="l",xlab="",ylab="")
lines(mat[,1],mat[,"bsup"],lty=1)
lines(mat[,1],mat[,"Naive"],lty=4,col=2)
lines(mat[,1],mat[,"CT"],lty=4,col=3)
lines(mat[,1],mat[,"ABW"],lty=4,col=4)
lines(mat[,1],mat[,"Beta"],lty=4,col="orange")
legend("bottomright",lty=c(1,4,4,4,4), col=c(1,2,3,4,"orange"), legend=(c("Identification bounds","Naive","Carrington-Troske","Allen et al","Rathelot")))
dev.off()


## Figure 2: Probability P constrained

mc1 = read.csv("simulation_mcontraint.csv",sep=",",head=FALSE)

library(tidyr)
library(ggplot2)
mc1$K <- 2:12

dat <- mc1 %>% gather(n, probability, V1:V4)

pdf("simulation_Pcontraint.pdf",height=5,width=7)
ggplot(dat, aes(K,probability, group=n, linetype=n))+
    geom_line()+
    scale_linetype_discrete(name="Number of units",labels=c("50","200","1,000","10,000"))+
    scale_x_continuous(name="Unit size",breaks=c(2,4,6,8,10,12))+
    scale_y_continuous(name=expression( Pr(tilde(P) != hat(P)) ))
dev.off()


