%
% Simulations: tables and figures of the paper
% Updated May 2016
% Xavier d'Haultfoeuille, Roland Rathelot
% 


%% Figure 1: sharp bounds and alternative solutions
%%%  for a simulated dataset, K by K

clear;
vect_K = 2:10;
nb_K= length(vect_K);
n = 1e6;

msig0 = [-3.12;1.56];
[true_Theil true_Duncan]=vrai_param(msig0)
db = -99*ones(nb_K,max(vect_K)+2);
res1 = zeros(nb_K,5);

for jj = 1:nb_K
    K=vect_K(jj);
    m = moment_normale_trans(msig0,K); 
    Q=fct_Q(K);
    Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];
    count = n*Pvrai;
    db(jj,1:length(count))=count;
    res1(jj,:) = bounds(db,K);
end

res2=alternatives_Kwise(db,vect_K);

reskwise = [vect_K' res1(:,2:5) res2]
dlmwrite('simu_Kwise_Fig1.csv', reskwise, '-append','delimiter' , ',')


%%% Table 1: sharp bounds and alternative solutions
%%%  for a simulated dataset, K pooled

clear;
vect_K = 2:10;
nb_K= length(vect_K);
n = 1e5;

msig0 = [-3.12;1.56];
[true_Theil true_Duncan]=vrai_param(msig0);
db = -99*ones(nb_K,max(vect_K)+2);

for jj = 1:nb_K
    K=vect_K(jj);
    m = moment_normale_trans(msig0,K); 
    Q=fct_Q(K);
    Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];
    count = n*Pvrai;
    db(jj,1:length(count))=count;
end

% Compute the bounds
[boundsW,boundsU]=bounds(db,vect_K)
% Compute the alternative methods (naive, Carrington-Troske, Allen et al.)
res3=alternatives_Kindep(round(db),vect_K)


%% Figure 2: Proportions of draws with estimated moments outside Mrond

clear
niter = 2e3; %number of simus
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters

vectK = 2:12;
vectn = [50 200 1000 10000];

resmat = -99*ones(size(vectK,2),size(vectn,2));

for in = 1:size(vectn,2)
    n = round(vectn(in));

    for iK = 1:size(vectK,2)
        K = vectK(iK);
        Q=fct_Q(K);
        % compute true moments and true probas
        m = moment_normale_trans(msig0,K);  
        Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];

        % simulate data in this dgp with multinomial draws
        contraint = ones(1,niter);
        tic;
        for ll = 1:niter;
            count = mnrnd(n,Pvrai);
            while count(1) == n %enlve les cas db = [n 0 0 0 ...]
                count = mnrnd(n,Pvrai);
            end
            n = round(sum(count));
            Pcomplet = count'/n;
            Pchap = Pcomplet(2:end);
            contraint(1,ll) = fct_contraint(Q \ Pchap);
        end
        toc;
        
        resmat(iK,in) = mean(contraint);
    end

end

dlmwrite('simulation_mcontraint.csv', resmat, '-append','delimiter' , ',')

%% Table 2 - Monte Carlo simulations: bounds, CIs, coverage rates
%%% K=3,6,9,1; n=1e2,1e3,1e4

clear
niter = 4e2; %number of sims
nboot = 2e2; %number of bootstraps
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
timemat = zeros(3,4); %measure time for each case
mmmat = -99*ones(10, niter , 4, 3);

for in = 1:3
    n = 10^(1+in);
    
    for iK = 1:4
        tic     
        Kset = 3*iK;
        mmat = -99*ones(10, niter);
        
        parfor ll = 1:niter 
            % simulation data
            dbm = - 99 * ones(length(Kset), max(Kset)+2);
            for iiK = 1:length(Kset)
                KK = Kset(iiK);
                Q=fct_Q(KK);
                m = moment_normale_trans(msig0,KK);  
                Pvrai = [1-sum(Q * m(1:KK)), (Q*m(1:KK))'];
                dbm(iiK,1:KK+1) = mnrnd(n,Pvrai);
                while dbm(iiK,1) == n %remove cases db = [n 0 0 0 ...]
                    dbm(iiK,1:KK+1) = mnrnd(n,Pvrai);
                end
            end
            % estimation of the index and CI
            tmp =  boundsCI(dbm, Kset, nboot);
            mmat(:,ll)= tmp(1,1:10);
        end        
        mmmat(:,:,iK,in) = mmat;
        timemat(in,iK) = toc;         
    end
end

save('simulations_CI_K36912.mat','mmmat')
timemat


% Compute quantities showed in Table 2
load('simulations_CI_K36912.mat')
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
[true_Theil, true_Duncan, vraicoworker]=vrai_param(msig0);

bestbornes_Theil = zeros(2,4,3); %2 * Nb K * Nb n
bestbornes_Duncan = zeros(2,4,3); %2 * Nb K * Nb n
moybornes_Theil = zeros(2,4,3); %2 * Nb K * Nb n
moybornes_Duncan = zeros(2,4,3); %2 * Nb K * Nb n
sigbornes_Theil = zeros(2,4,3); %2 * Nb K * Nb n
sigbornes_Duncan = zeros(2,4,3); %2 * Nb K * Nb n
lengthCI1_Theil = zeros(1,4,3); %2 * Nb K * Nb n
lengthCI1_Duncan = zeros(1,4,3); %2 * Nb K * Nb n
CR1_Theil = zeros(1,4,3); %2 * Nb K * Nb n
CR1_Duncan = zeros(1,4,3); %2 * Nb K * Nb n
for in = 1:3
    n = 10^(1+in);
    for iK = 1:4
        mmat = mmmat(:,:,iK,in);
        K = 3*iK;
        k=floor(K/2)+1;
        xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;    
        Q=fct_Q(K);
        % compute true moments, true probas and true indices
        m = moment_normale_trans(msig0,K);  
        Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];
        bornes_Theil = estimation_bornes_Theil_cheb(m, xp0, 0);
        %bornes_Duncan = estimation_bornes_Duncan_interior(m);
        bornes_Duncan = estimation_bornes_Duncan_non_lin(m,xp0,0);
        disp 'Nb of units and nb of indiv per unit (n,k)'
        [n,K]
        disp 'True Theil and Duncan'
        [bornes_Theil,bornes_Duncan]
        IC_Theil = [mmat(7,:)', mmat(8,:)'];
        IC_Duncan = [mmat(9,:)',mmat(10,:)'];
        indic_dedans_Theil = ((true_Theil>=IC_Theil(:,1)) & (true_Theil<=IC_Theil(:,2)));
        indic_dedans_Duncan = ((true_Duncan>=IC_Duncan(:,1)) & (true_Duncan<=IC_Duncan(:,2)));
        disp 'Average Theil and Duncan'
        [mean(mmat(2:3,:)'), mean(mmat(4:5,:)')]
        disp 'Taux de couverture du Theil et du Duncan'
        [mean(indic_dedans_Theil), mean(indic_dedans_Duncan)]
        %Assignation
        bestbornes_Theil(:,iK,in)=real(bornes_Theil);
        bestbornes_Duncan(:,iK,in)=real(bornes_Duncan);
        moybornes_Theil(:,iK,in)=mean(mmat(2:3,:),2);
        moybornes_Duncan(:,iK,in)=mean(mmat(4:5,:),2);
        sigbornes_Theil(:,iK,in)=std(mmat(2:3,:),0,2);
        sigbornes_Duncan(:,iK,in)=std(mmat(4:5,:),0,2);
        lengthCI1_Theil(1,iK,in)=mean(mmat(8,:)-mmat(7,:),2);
        lengthCI1_Duncan(1,iK,in)=mean(mmat(10,:)-mmat(9,:),2);
        CR1_Theil(1,iK,in)=mean(indic_dedans_Theil);
        CR1_Duncan(1,iK,in)=mean(indic_dedans_Duncan);
    end
end

bT = permute(moybornes_Theil,[3 2 1]);
bT1 = reshape(bT(:,:,1), [12,1,1]);
bT2 = reshape(bT(:,:,2), [12,1,1]);

bD = permute(moybornes_Duncan,[3 2 1]);
bD1 = reshape(bD(:,:,1), [12,1,1]);
bD2 = reshape(bD(:,:,2), [12,1,1]);

sT = permute(sigbornes_Theil,[3 2 1]);
sT1 = reshape(sT(:,:,1), [12,1,1]);
sT2 = reshape(sT(:,:,2), [12,1,1]);

sD = permute(sigbornes_Duncan,[3 2 1]);
sD1 = reshape(sD(:,:,1), [12,1,1]);
sD2 = reshape(sD(:,:,2), [12,1,1]);

lT = reshape(permute(lengthCI1_Theil,[3 2 1]), [12,1,1]);
lD = reshape(permute(lengthCI1_Duncan,[3 2 1]), [12,1,1]);

cT = reshape(permute(CR1_Theil,[3 2 1]), [12,1,1]);
cD = reshape(permute(CR1_Duncan,[3 2 1]), [12,1,1]);

resmat = [[sT1 bT1 sT2 bT2 lT cT];[sD1 bD1 sD2 bD2 lD cD]]

%%% Table 2 (second panel)
%%% Monte Carlo simulations: bounds, CIs, coverage rates
%%% K random; n=1e2,1e3,1e4; weighted and unweighted

clear
vect_K = [3 6 9 12];
niter = 2e2; %number of sims
nboot = 2e2; %number of bootstrap to compute CI
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
mnorm = moment_normale_trans(msig0,max(vect_K));

mmmat = -99*ones(2,10,niter,3);

% Took 650,000 sec for 400 simulations x 200 bootstrap (parfor)
% 6,500 sec for 100 simus x 50 bootstrap
% 84,000 sec for 200 simus x 100 bootstrap
% 7,800 sec for 400 simus x 10 bootstrap
% 54,000 for 100 simus x 200 boot

tic
for in = 1:3
    n = 10^(1+in);
    mmat = -99*ones(2,10,niter);
    parfor ll = 1:niter 
        db = -99*ones(length(vect_K),max(vect_K)+2);
        for iKK = 1:length(vect_K)	
            KK = vect_K(iKK);
            m = mnorm(1:KK);  
            Q=fct_Q(KK);
            Pvrai = [1-sum(Q * m(1:KK)), (Q*m(1:KK))'];
            count = mnrnd(n,Pvrai);
            while count(1) == n %remove cases where db = [n 0 0 0 ...]
                count = mnrnd(n,Pvrai);
            end
            db(iKK,1:length(count))=count;
        end    
        tmp = boundsCI(db, vect_K, nboot); 
        mmat(:,:,ll) = tmp(:,1:10);
    end
    mmmat(:,:,:,in)= mmat; 
end
toc

save('simulations_CI_Krandom.mat','mmmat')

% Compute quantities showed in Table 
vect_K = [3 6 9 12];
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
[true_Theil, true_Duncan]=vrai_param(msig0)
mnorm = moment_normale_trans(msig0,max(vect_K));

db = -99*ones(length(vect_K),max(vect_K)+2);
for iK = 1:length(vect_K)	
    K = vect_K(iK);
    m = mnorm(1:K);  
    Q=fct_Q(K);
    Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];
    count = 1e6*Pvrai;
    db(iK,1:length(count))=count;
end    

[resb,resbU] = bounds(db,vect_K);
ident_Theil = [resb(2:3);resbU(2:3)];
ident_Duncan = [resb(4:5);resbU(4:5)];

bestbornes_Theil = zeros(2,2,3); %2 * weight/unweight * Nb n
bestbornes_Duncan = zeros(2,2,3); %2 * weight/unweight * Nb n
moybornes_Theil = zeros(2,2,3); %2 * weight/unweight * Nb n
moybornes_Duncan = zeros(2,2,3); %2 * weight/unweight * Nb n
sigbornes_Theil = zeros(2,2,3); %2 * weight/unweight * Nb n
sigbornes_Duncan = zeros(2,2,3); %2 * weight/unweight * Nb n
lengthCI1_Theil = zeros(1,2,3); %2 * weight/unweight * Nb n
lengthCI1_Duncan = zeros(1,2,3); %2 * weight/unweight * Nb n
CR1_Theil = zeros(1,2,3); %2 * weight/unweight * Nb n
CR1_Duncan = zeros(1,2,3); %2 * weight/unweight * Nb n

load('simulations_CI_Krandom.mat')
for icase = 1:2   
    bornes_Theil = ident_Theil(icase,:);
    bornes_Duncan = ident_Duncan(icase,:);
    for in = 1:3
        n = 10^(1+in); 
        mmat = permute(mmmat(icase,:,:,in),[2 3 1]);
        disp 'Nb of units and case'
        [icase,n]
        disp 'True Theil and Duncan'
        [bornes_Theil,bornes_Duncan]
        IC_Theil = [mmat(7,:)', mmat(8,:)'];
        IC_Duncan = [mmat(9,:)',mmat(10,:)'];
        indic_dedans_Theil = ((true_Theil>=IC_Theil(:,1)) & (true_Theil<=IC_Theil(:,2)));
        indic_dedans_Duncan = ((true_Duncan>=IC_Duncan(:,1)) & (true_Duncan<=IC_Duncan(:,2)));
        disp 'Average Theil and Duncan'
        [mean(mmat(2:3,:)'), mean(mmat(4:5,:)')]
        disp 'Coverage rate Theil and Duncan'
        [mean(indic_dedans_Theil), mean(indic_dedans_Duncan)]
        %Assignation
        bestbornes_Theil(:,icase,in)=real(bornes_Theil);
        bestbornes_Duncan(:,icase,in)=real(bornes_Duncan);
        moybornes_Theil(:,icase,in)=mean(mmat(2:3,:),2);
        moybornes_Duncan(:,icase,in)=mean(mmat(4:5,:),2);
        sigbornes_Theil(:,icase,in)=std(mmat(2:3,:),0,2);
        sigbornes_Duncan(:,icase,in)=std(mmat(4:5,:),0,2);
        lengthCI1_Theil(1,icase,in)=mean(mmat(8,:)-mmat(7,:),2);
        lengthCI1_Duncan(1,icase,in)=mean(mmat(10,:)-mmat(9,:),2);
        CR1_Theil(1,icase,in)=mean(indic_dedans_Theil);
        CR1_Duncan(1,icase,in)=mean(indic_dedans_Duncan);
    end
end

bT = permute(moybornes_Theil,[3 2 1]);
bT1 = reshape(bT(:,:,1), [6,1,1]);
bT2 = reshape(bT(:,:,2), [6,1,1]);

bD = permute(moybornes_Duncan,[3 2 1]);
bD1 = reshape(bD(:,:,1), [6,1,1]);
bD2 = reshape(bD(:,:,2), [6,1,1]);

sT = permute(sigbornes_Theil,[3 2 1]);
sT1 = reshape(sT(:,:,1), [6,1,1]);
sT2 = reshape(sT(:,:,2), [6,1,1]);

sD = permute(sigbornes_Duncan,[3 2 1]);
sD1 = reshape(sD(:,:,1), [6,1,1]);
sD2 = reshape(sD(:,:,2), [6,1,1]);

lT = reshape(permute(lengthCI1_Theil,[3 2 1]), [6,1,1]);
lD = reshape(permute(lengthCI1_Duncan,[3 2 1]), [6,1,1]);

cT = reshape(permute(CR1_Theil,[3 2 1]), [6,1,1]);
cD = reshape(permute(CR1_Duncan,[3 2 1]), [6,1,1]);

resmat = [[sT1 bT1 sT2 bT2 lT cT];[sD1 bD1 sD2 bD2 lD cD]]

%% Table 3: CPU Time 
%%% K=3,6,9,12,20; n=1e2 or infty; weighted index

clear
timemat = zeros(5,5); %measure time for each case and each quantity
contraint_sum = zeros(1,5);
msig0 = [-3.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
Kset = [3 6 9 12 20];

% Constrained ML
% The True DGP is fixed and we draw P in a  MNRND around Pvrai
niter = 1e2; %number of simus
n = 100; 
for iK = 1:5
    K = Kset(iK);
    Q=fct_Q(K);
    % compute true moments and true probas
    m = moment_normale_trans(msig0,K);  
    Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];
    % simulate data in this dgp with multinomial draws
    dbb = - 99 * ones(niter, K+2);
    for ll = 1:niter;
        dbb(ll,1:K+1) = mnrnd(n,Pvrai);
        while dbb(1) == n %remove cases db = [n 0 0 0 ...]
            dbb(ll,1:K+1) = mnrnd(n,Pvrai);
        end
    end
    % now, the estimation begins 
    for ll = 1:niter;
        db = dbb(ll,:);
        k=floor(K/2)+1;
        % Manage data
        count = db(db>-99);
        n = round(sum(count));
        Pcomplet = count'/n;
        Pchap = Pcomplet(2:end);
        % Initial support (discrete uniform distribution with k points)
        xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k];
        % is mchap in Mrond?
        Q=fct_Q(K);   
        mtilde = Q \ Pchap;
        contraint = fct_contraint(mtilde);
        contraint_sum(1,iK) = contraint_sum(1,iK) + contraint;

        tic;
        % constrained ML
        if contraint == 1
            [xp1,~] = projection_xp(Pchap, xp0); 
            [xp1,Pchap2_complet] = projection_maxvrais(Pchap, xp1); 
            Pchap2 = Pchap2_complet(2:end);
        else
            xp1 = xp0;
            Pchap2_complet = Pcomplet;
            Pchap2 = Pchap;
        end
        timemat(1,iK) = timemat(1,iK)+toc; 
    end 
end 
contraint_sum
timemat(1,:)
niter*timemat(1,:)./contraint_sum

% Theil (Chebyshev and normal) and Duncan bounds
% In order to stay in Mrond, we draw the params of the true DGP

niter = 1e2; %number of simus
n=1e4;
for iK = 1:5
    K = Kset(iK);
    Q=fct_Q(K);    
    % simulate data in this dgp with multinomial draws
    dbb = - 99 * ones(niter, K+2);
    for ll = 1:niter;
       msig = msig0 + rand(2,1)/100-.005;
       m = moment_normale_trans(msig,K);  
       Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))']; 
       dbb(ll,1:K+1) = n*Pvrai;
    end
    
    % now, the estimation begins
    % compute the indices and CI
    mmat = -99*ones(15, niter);
    for ll = 1:niter;
        db = dbb(ll,:);
        k=floor(K/2)+1;
        count = db(db>-99);
        n = round(sum(count));
        Pcomplet = count'/n;
        Pchap = Pcomplet(2:end);
        xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;
        Q=fct_Q(K);   
        mtilde = Q \ Pchap;
        contraint = fct_contraint(mtilde);
        % constrained ML
        if contraint == 1
            [xp1,~] = projection_xp(Pchap, xp0); 
            [xp1,Pchap2_complet] = projection_maxvrais(Pchap, xp1); 
            Pchap2 = Pchap2_complet(2:end);
        else
            xp1 = xp0;
            Pchap2_complet = Pcomplet;
            Pchap2 = Pchap;
        end
        % Bounds computation
        tic;
        bornes_Theil = estimation_bornes_Theil_cheb(mtilde, xp1, contraint);
        timemat(2,iK) = timemat(2,iK)+toc;  
        
        tic;
        bornes_Theil = estimation_bornes_Theil_non_lin(mtilde, xp1, contraint);
        timemat(3,iK) = timemat(3,iK)+toc;  
        
        tic;
        bornes_Duncan = estimation_bornes_Duncan_non_lin(mtilde, xp1, contraint);
        timemat(4,iK) = timemat(4,iK)+toc;         
    end
end
timemat(2:4,:)


% Confidence intervals
%
niter = 1e2; %number of simus
n = 1e4;
for iK = 1:5
    K = Kset(iK);
    Q=fct_Q(K);
    % compute true moments and true probas
    m = moment_normale_trans(msig0,K);  
    Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];    
    % simulate data in this dgp with multinomial draws
    dbb = - 99 * ones(niter, K+2);
    for ll = 1:niter;
        dbb(ll,1:K+1) = mnrnd(n,Pvrai);
        while dbb(1) == n %remove cases db = [n 0 0 0 ...]
            dbb(ll,1:K+1) = mnrnd(n,Pvrai);
        end
    end    
    % now, the estimation begins
    mmat = -99*ones(15, niter);
    for ll = 1:niter;
        db = dbb(ll,:);
        k=floor(K/2)+1;
        count = db(db>-99);
        n = round(sum(count));
        Pcomplet = count'/n;
        Pchap = Pcomplet(2:end);
        xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;
        Q=fct_Q(K);   
        mtilde = Q \ Pchap;
        contraint = fct_contraint(mtilde);
        if contraint == 1
            [xp1,~] = projection_xp(Pchap, xp0); 
            [xp1,Pchap2_complet] = projection_maxvrais(Pchap, xp1); 
            Pchap2 = Pchap2_complet(2:end);
        else
            xp1 = xp0;
            Pchap2_complet = Pcomplet;
            Pchap2 = Pchap;
        end
        % Bounds
        bornes_Theil = estimation_bornes_Theil_cheb(mtilde, xp1, contraint);
        bornes_Duncan = estimation_bornes_Duncan_non_lin(mtilde, xp1, contraint);        
        % Confidence intervals
        tic;
        CI = fct_CI1(Pchap2_complet,xp1,contraint,n,Q,bornes_Theil,bornes_Duncan,100,.05,1);
        timemat(5,iK) = timemat(5,iK)+toc; 
    end
end
max(contraint_sum)*timemat(1,:)./contraint_sum
contraint_sum
timemat


%% Table A1: Alternative DGPs
%%% Monte Carlo simulations: bounds, CIs, coverage rates
%%% K=6; n=1e2,1e3,1e4
clear
K = 6;
niter = 4e2; %number of simus
nboot = 2e2; %number of bootstraps
Q=fct_Q(K);

%%% DGPs
mdgp1 = moment_normale_trans([-2.12;1.56],K); %dgp Phi^{-1}(p) normal with these parameters
mdgp2 = moment_normale_trans([-1.12;1.56],K); %dgp Phi^{-1}(p) normal with these parameters
x = [0;1/3];
p = [0.9;0.1];
mdgp3 = ((ones(K,1) * x' ).^((1:K)' * ones(1,length(x)))) * p;
mdgp = [mdgp1 mdgp2 mdgp3];

%%% Create data and estimate bounds and CI
tic
mmmat = -99*ones(10, niter , 3, 3);  
for in = 1:3
    n = 10^(1+in);
    for idgp = 1:3        
        m = mdgp(:,idgp);
        Pvrai = [1-sum(Q * m(1:K)), (Q*m(1:K))'];	
        mmat = -99*ones(10, niter);  
        parfor ll = 1:niter 
            % simulation data
            dbm = - 99 * ones(length(K), max(K)+2);
            dbm(1,1:K+1) = mnrnd(n,Pvrai);
            while dbm(1,1) == n %enlève les cas db = [n 0 0 0 ...]
                dbm(1,1:K+1) = mnrnd(n,Pvrai);
            end
            % estimation of the index and CI
            tmp= boundsCI(dbm, K, nboot);
            mmat(:,ll)= tmp(1,1:10);
        end
        mmmat(:,:,idgp,in) = mmat;
        toc
    end
end

save('simulations_CI_altdgp.mat','mmmat')

% Compute quantities showed in Table 
K=6;
k=floor(K/2)+1;
xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;    
Q=fct_Q(K);
        
bestbornes_Theil = zeros(2,3,3); %2 * Nb dgp * Nb n
bestbornes_Duncan = zeros(2,3,3); %2 * Nb dgp * Nb n
moybornes_Theil = zeros(2,3,3); %2 * Nb dgp * Nb n
moybornes_Duncan = zeros(2,3,3); %2 * Nb dgp * Nb n
sigbornes_Theil = zeros(2,3,3); %2 * Nb dgp * Nb n
sigbornes_Duncan = zeros(2,3,3); %2 * Nb dgp * Nb n
lengthCI1_Theil = zeros(1,3,3); %2 * Nb dgp * Nb n
lengthCI1_Duncan = zeros(1,3,3); %2 * Nb dgp * Nb n
CR1_Theil = zeros(1,3,3); %2 * Nb dgp * Nb n
CR1_Duncan = zeros(1,3,3); %2 * Nb dgp * Nb n

load('simulations_CI_altdgp.mat')
for in = 1:3
    n = 10^(1+in);
    for idgp = 1:3
    
        if idgp == 1
        msig0 = [-2.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
        [true_Theil, true_Duncan]=vrai_param(msig0);
        end
    
        if idgp == 2
        msig0 = [-1.12;1.56]; %dgp Phi^{-1}(p) normal with these parameters
        [true_Theil, true_Duncan]=vrai_param(msig0);
        end
        
        if idgp == 3
        true_Theil = .677; true_Duncan=.931;
        end    
    
        m = mdgp(:,idgp);
        mmat = mmmat(:,:,idgp,in);
        % compute true moments, true probas and true indices
        bornes_Theil = estimation_bornes_Theil_cheb(m, xp0, 0);
        bornes_Duncan = estimation_bornes_Duncan_non_lin(m,xp0,0);
        disp 'Nb of units and nb of indiv per unit (n,k)'
        [n,K]
        disp 'True Theil and Duncan'
        [bornes_Theil,bornes_Duncan]
        IC_Theil = [mmat(7,:)', mmat(8,:)'];
        IC_Duncan = [mmat(9,:)',mmat(10,:)'];
        indic_dedans_Theil = ((true_Theil>=IC_Theil(:,1)) & (true_Theil<=IC_Theil(:,2)));
        indic_dedans_Duncan = ((true_Duncan>=IC_Duncan(:,1)) & (true_Duncan<=IC_Duncan(:,2)));
        disp 'Average Theil and Duncan'
        [mean(mmat(2:3,:)'), mean(mmat(4:5,:)')]
        disp 'Taux de couverture du Theil et du Duncan'
        [mean(indic_dedans_Theil), mean(indic_dedans_Duncan)]
        %Assignation
        bestbornes_Theil(:,idgp,in)=real(bornes_Theil);
        bestbornes_Duncan(:,idgp,in)=real(bornes_Duncan);
        moybornes_Theil(:,idgp,in)=mean(mmat(2:3,:),2);
        moybornes_Duncan(:,idgp,in)=mean(mmat(4:5,:),2);
        sigbornes_Theil(:,idgp,in)=std(mmat(2:3,:),0,2);
        sigbornes_Duncan(:,idgp,in)=std(mmat(4:5,:),0,2);
        lengthCI1_Theil(1,idgp,in)=mean(mmat(8,:)-mmat(7,:),2);
        lengthCI1_Duncan(1,idgp,in)=mean(mmat(10,:)-mmat(9,:),2);
        CR1_Theil(1,idgp,in)=mean(indic_dedans_Theil);
        CR1_Duncan(1,idgp,in)=mean(indic_dedans_Duncan);
    end
end

bT = permute(moybornes_Theil,[3 2 1]);
bT1 = reshape(bT(:,:,1), [9,1,1]);
bT2 = reshape(bT(:,:,2), [9,1,1]);

bD = permute(moybornes_Duncan,[3 2 1]);
bD1 = reshape(bD(:,:,1), [9,1,1]);
bD2 = reshape(bD(:,:,2), [9,1,1]);

sT = permute(sigbornes_Theil,[3 2 1]);
sT1 = reshape(sT(:,:,1), [9,1,1]);
sT2 = reshape(sT(:,:,2), [9,1,1]);

sD = permute(sigbornes_Duncan,[3 2 1]);
sD1 = reshape(sD(:,:,1), [9,1,1]);
sD2 = reshape(sD(:,:,2), [9,1,1]);

lT = reshape(permute(lengthCI1_Theil,[3 2 1]), [9,1,1]);
lD = reshape(permute(lengthCI1_Duncan,[3 2 1]), [9,1,1]);

cT = reshape(permute(CR1_Theil,[3 2 1]), [9,1,1]);
cD = reshape(permute(CR1_Duncan,[3 2 1]), [9,1,1]);

resmat = [[sT1 bT1 sT2 bT2 lT cT];[sD1 bD1 sD2 bD2 lD cD]]

%% Table A2: Test of the binomial model
%%% Monte Carlo simulations: true levels of test
%%% K=3,6,9,12; n=1e2,1e3,1e4

clear
niter = 2e3; %nb of simulations
nboot = 2e2; %nb of bootstraps
nset = [100,1000,10000]; %nb of units
vect_K = [3,6,9,12];

% DGP
x = [0;1/3];
p = [0.9;0.1];

pvalue = zeros(length(nset),length(vect_K),niter);
tic
for in=1:length(nset)
    n = nset(in);
    for ik=1:length(vect_K)
        K=vect_K(ik);
        Q=fct_Q(K);
        m_vrai = ((ones(K,1) * x' ).^((1:K)' * ones(1,length(x)))) * p;
        P = Q * m_vrai;        
        Mat_Ptilde = -99 *ones(niter,K+2);        
        Mat_Ptilde(:,1:K+1) = mnrnd(n,[1-sum(P),P'],niter);        
        pvaltmp = -99*ones(niter,1);
        parfor s=1:niter 
            pvaltmp(s,1) = fct_test_binomiale_bootmv(Mat_Ptilde(s,:),K,nboot);
        end
        pvalue(in,ik,:) = pvaltmp;
        
    end
end
toc

save('pvaltestbinomial.mat','pvalue')
mean(pvalue<.05,3)