%
% Test of constance of the segregation indices across K
% Requires: diff_for_test_constant_segreg.m
% Updated May 2016
% Xavier d'Haultfoeuille and Roland Rathelot
% 

function pvalue = test_constant_segreg(db, vect_K, indic_theil)

% Nb of subsamples
S=5e2;
% Sampling rate
f = 0.1;

Ksup = max(vect_K);
if size(vect_K,1)>1
    vect_K=vect_K';
end
n_K = length(vect_K);
vect_n = zeros(n_K,1);
mat_seuils = zeros(n_K,Ksup+1);
for j = 1:n_K
    K = vect_K(j);
    [val,iiii] = min(db');
    count = db( (iiii-2 == K)&(val==-99) ,:);
    count = count(count>-99);
    vect_n(j) = round(sum(count));
    mat_seuils(j,1:K+1) = cumsum(count);
end

n_tot = sum(vect_n);
vect_b_n = round(f*vect_n);
b_n_tot = sum(vect_b_n);

% Test stat
diff = diff_for_test_constant_segreg(db, vect_K, indic_theil);
stat_test = sqrt(n_tot) * diff;

% Subsampling to get the p-value
diff_ss_ech = zeros(S,1);
for s=1:S
    % subsampling
    db_ss_ech = -99 * ones(n_K,Ksup+2);
    for j = 1:n_K
        K = vect_K(j);
        n = vect_n(j);
        b_n = vect_b_n(j);
        A = eye(K+1) - diag(ones(K,1),1);
        seuils = mat_seuils(j,1:K+1);
        perm_rand = randperm(n, b_n);
        pr_ss_ech=sum((bsxfun(@minus,perm_rand',seuils)<=0));
        db_ss_ech(j,1:K+1) = pr_ss_ech * A;
    end

    % test stat on the subsample
    diff_ss_ech(s) = diff_for_test_constant_segreg(db_ss_ech, vect_K, indic_theil);
end
vect_stat_ss_ech = sqrt(b_n_tot) * (diff_ss_ech - diff);
pvalue = mean((vect_stat_ss_ech<=stat_test));

end

function diff = diff_for_test_constant_segreg(db, vect_K, indic_theil)

terme_max=-Inf;
terme_min = Inf;

for j = 1:length(vect_K)
    
    K=vect_K(j);
    k=floor(K/2)+1;
    
    % Gere les donnees
    [val,iiii] = min(db');
    count = db( (iiii-2 == K)&(val==-99) ,:);
    count = count(count>-99);
    n = round(sum(count));
    Pcomplet = count'/n;
    Pchap = Pcomplet(2:end);
    
    % Support initial (rpartition uniforme des K+1 points)
    xp0 = [(1/(k+1):1/(k+1):(k)/(k+1))';ones(k,1)/k] ;
    
    % mchap est-il dans Mrond    
    Q=fct_Q(K);   
    mtilde = Q \ Pchap;
    contraint = fct_contraint(mtilde);
       
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    % 2. Calcul des bornes de la rgion d'identification
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Cas mchap  l'extrieur (quel xp projet dans Mrond)
    if contraint == 1
        [xp1,~] = projection_xp(Pchap, xp0); 
        [xp1,~] = projection_maxvrais(Pchap, xp1); 
    else
        xp1 = xp0;
    end
     
    % Calcul des bornes
    if indic_theil == 1 
        bornes = estimation_bornes_Theil_cheb(mtilde, xp1, contraint); 
    else 
        bornes = estimation_bornes_Duncan_non_lin(mtilde, xp1, contraint); 
    end

    terme_max = max(terme_max, bornes(1));
    terme_min = min(terme_min, bornes(2));
end

diff = terme_min - terme_max;

end